#include "Ball.h"

Ball::Ball() : cBall(0,0,0), vSpeed()
{   // constructor
    uColorHigh=Color565::fromRGB(0xFF, 66, 66);
    uColorMid=Color565::fromRGB(0xBB, 22, 22);
    uColorLow=Color565::fromRGB(0x88, 0, 0);
}

void Ball::initialize(LCD_ST7735* pDisp, int nX, int nY, int nR)
{
    this->pDisp=pDisp;
    this->pos.set(nX, nY);
    this->cBall=Circle(nX, nY, nR);
}

void Ball::setSpeed(int X, int Y)
{
    this->vSpeed.set(X, Y);
}

void Ball::changeSpeed(bool fUp)
{
    if(fUp)
    {
        if(this->vSpeed.getSize()<=4.0) this->vSpeed.add(1.0);
    }
    else
    {
        if(this->vSpeed.getSize()>=1.0) this->vSpeed.add(-1.0);
    }
}

void Ball::update()
{
    this->pos.move(this->vSpeed);
    this->cBall.setXY(this->pos.getX(), this->pos.getY());
}

Circle Ball::getBoundingCircle()
{
    return(cBall);
}

bool Ball::collides(Rectangle r)
{
    Rectangle rBall=this->cBall.getBoundingRectangle();

/*    
char szBuffer[256];
sprintf(szBuffer, "c:%d,%d      ", cBall.getX(), cBall.getY());
this->pDisp->drawString(font_ibm, 0, 0, szBuffer);
*/
    return(rBall.collides(r));
}

void Ball::Bounce(Vector vBounce)
{   // change the direction in a certain direction
    this->vSpeed.multiply(vBounce);

    // check speed w/max
    if(this->vSpeed.y>5.0) this->vSpeed.y=5;
}



void Ball::clear()
{
    Point p=pos.getCur();
    this->pDisp->fillCircle(p.getX(),p.getY(), cBall.getRadius(), Color565::Black, Color565::Black);
}

void Ball::clearPrev()
{
    Point p=pos.getPrev();
    this->pDisp->fillCircle(p.getX(),p.getY(), cBall.getRadius(), Color565::Black, Color565::Black);
}


void Ball::draw()
{   // render the object on the screen, based on its current position
    Point p=pos.getCur();
    if(cBall.getRadius()>3)
    {
        this->pDisp->fillCircle(p.getX(), p.getY(), cBall.getRadius(), this->uColorLow, this->uColorLow);
        this->pDisp->fillCircle(p.getX()-cBall.getRadius()/3, p.getY()-cBall.getRadius()/3, cBall.getRadius()/2, this->uColorMid, this->uColorMid);
        this->pDisp->setPixel(p.getX()-cBall.getRadius()/2, p.getY()-cBall.getRadius()/2, this->uColorHigh);
    }
    else
        this->pDisp->fillCircle(p.getX(), p.getY(), cBall.getRadius(), this->uColorMid, this->uColorMid);
}

void Ball::redraw()
{   // redraw the ball if its position has changed
    if(pos.hasChanged())
    {
        clearPrev();
        draw();
    }
}